#---a popup window for choicing directories
# -initialdir "<start in dir> | home-dir (default)"
# -multiple "0 (default) | 1 (selecting of multiple dirs allowed)"
# -view_hidden_dirs "0 (default) | 1"
# -view_hidden_files "0 (default) | 1"
# -title "<your window title> | "Please choose a directory:" (default)"
# -parent "<window which starts FR_dirBox> | . (default)"
# -permission "writable | readable (default)"
# -balloonhelp "0 (default) | 1"
# -filetype "[list "Description" "*.file extension"] | * (default)"

package provide FR_dirBox 0.2

foreach Arg [list FR_messageBox FR_popupWindow FR_general FR_balloonHelp] \
{
 package require $Arg
}

namespace eval ::FR_dirBox \
{
 namespace export Create
}


#---the content of a directory
proc ::FR_dirBox::DirTreeContent {Path Dir} \
{
 variable VAR
 
 set Content [glob -directory $Dir -types {d} -tails -nocomplain *]
 
 if {$VAR($Path,HiddenDirs)} \
 {
  set Tmp [glob -directory $Dir -types {d hidden} -tails -nocomplain *]
  set Tmp [lrange $Tmp 2 end]
  eval lappend Content $Tmp
 }
 
 set Content [lsort -dictionary $Content]
   
 if { $Dir ne "/" } { set Content [linsert $Content 0 ".."] }
   
 return $Content
}


#---fills the directories into the listbox
proc ::FR_dirBox::DirTreeView {Path Dir} \
{   
 variable VAR
   
 set Content [::FR_dirBox::DirTreeContent $Path $Dir]
    
 .$VAR($Path,dirtree).dirtree delete 0 end
   
 foreach Arg $Content \
 {
  .$VAR($Path,dirtree).dirtree insert end $Arg
 }
}


#---find the file type of the combobox and return it with glob pattern
proc ::FR_dirBox::ShortFind {Arg} \
{
 set ShortFound ""
 #---we are looking for *.(letters or numbers or *)
 set ShortFoundTmp [regexp -all -inline -- {\*.([A-Za-z0-9\*]+)} $Arg]
  
 #---we sort out the search return with the whole line
 for {set i 0} {$i < [llength $ShortFoundTmp]} {incr i 2} \
 {
  lappend ShortFound [lindex $ShortFoundTmp $i]
 }
  
 #---we replace e.g *.wav with *.[Ww][Aa][Vv]
 for {set i 0} {$i < [llength $ShortFound]} {incr i} \
 {
  set NewArg ""
  set Arg [lindex $ShortFound $i]
   
  #---in case of *.* we set it to * to fetch files without tail also (e.g. Readme)
  if {$Arg eq "*.*"} \
  {
   set NewArg "*"
  } else \
  {
   for {set j 0} {$j < [string length $Arg]} {incr j} \
   {
    set Letter [string index $Arg $j]
    
    if {[string is lower $Letter]} \
    {
     set Letter2 [string toupper $Letter]
     set NewLetter "\[${Letter2}${Letter}\]"
     append NewArg $NewLetter 
    } \
    elseif {[string is upper $Letter]} \
    {
     set Letter2 [string tolower $Letter]
     set NewLetter "\[${Letter}${Letter2}\]"
     append NewArg $NewLetter 
    } else \
    {
     append NewArg $Letter
    }
   }
  }
   
  lset ShortFound $i $NewArg
 }
  
 return $ShortFound
} 


#---returns the files in "dir"
proc ::FR_dirBox::FileTreeContent {Path Dir} \
{
  variable VAR
  
  set FileType [::FR_dirBox::ShortFind $VAR($Path,FileType)]
  
  set Content ""
  
  foreach Arg  $FileType \
  {
    set GlobReturnValue [glob -directory $Dir -types {f} -tails -nocomplain $Arg]
    if {$GlobReturnValue ne ""} \
    {
      eval lappend Content $GlobReturnValue
    }
  }
  
  if {$VAR($Path,HiddenFiles)} \
  {
    foreach Arg  $FileType \
    {
      set GlobReturnValue [glob -directory $Dir -types {hidden f} -tails -nocomplain $Arg]
   
      if {$GlobReturnValue ne ""} \
      {
        eval lappend Content $GlobReturnValue
      }
    }
  }
  
  set Content [lsort -dictionary $Content]
 
  return $Content 
}


#---fills the files into the listbox
proc ::FR_dirBox::FileTreeView {Path Dir} \
{   
 variable VAR
   
 set Content [::FR_dirBox::FileTreeContent $Path $Dir]
 
 .$VAR($Path,files).files configure -state normal
  
 .$VAR($Path,files).files delete 0 end  
  
 foreach Arg $Content \
 {
   .$VAR($Path,files).files insert end $Arg
 }
 
 .$VAR($Path,files).files configure -state disable
}


#---fills the chosen element into the entry-box
proc ::FR_dirBox::InsertText {Path Dir} \
{
 variable VAR
  
 .$VAR($Path,entry).entry configure -state normal
  
 .$VAR($Path,entry).entry delete 0 end
 .$VAR($Path,entry).entry insert end $Dir
  
 .$VAR($Path,entry).entry configure -state disabled
}


#---refresh the view
proc ::FR_dirBox::DirTreeRefresh {Path {Dir ""}} \
{
 variable VAR
 
 if {$Dir ne ""} \
 {
  eval set VAR($Path,TmpDir) $Dir
 }
 
 ::FR_dirBox::InsertText $Path $VAR($Path,TmpDir)
    
 ::FR_dirBox::DirTreeView $Path $VAR($Path,TmpDir)
 ::FR_dirBox::FileTreeView $Path $VAR($Path,TmpDir)
} 


#---the reaction of the double click on a directory
proc ::FR_dirBox::DirTreeSelection {Path Widget} \
{
 variable VAR
 
 set Index [$Widget curselection]
 if {$Index eq ""} {return}
    
 set Selected [$Widget get $Index]
   
 if { $Selected ne ".." } \
 {
  set VAR($Path,TmpDir) [file join $VAR($Path,TmpDir) $Selected]
 } else \
 {
  set LastSlash [string last / $VAR($Path,TmpDir)]
     
  #---to avoid the cutting of the / of the root-directory
  if { $LastSlash > 1} {incr LastSlash -1}
     
  set VAR($Path,TmpDir) [string range $VAR($Path,TmpDir) 0 $LastSlash]
 }
  
 ::FR_dirBox::DirTreeRefresh $Path $VAR($Path,TmpDir)
}
 
 
#---we check the needed permissions for this directory
proc ::FR_dirBox::RightsCheck {Path Dir} \
{
 variable VAR
  
 if {$VAR($Path,Permission) eq "writable"} \
 { 
  set Cmd [list file writable $Dir]
  set Text [msgcat::mc "write-"]
 } else \
 {
  set Cmd [list file readable $Dir]
  set Text [msgcat::mc "read-"]
 }
     
 if {![eval $Cmd]} \
 {
  ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "You need the %spermission for this directory, but you havn't it." $Text]     
  return 0
 } else \
 {
  return 1
 }
}
 
 
#---we read out the entry widget and return its content
proc ::FR_dirBox::EntryCheck {Path} \
{
 variable VAR
   
 set TestedDirs ""
    
 set DirList [.$VAR($Path,entry).entry get]
  
 foreach Arg $DirList \
 {
  if {[::FR_dirBox::RightsCheck $Path $Arg]} \
  {
   # we mask the spaces
   regsub -all {\s} $Arg {\\ } Arg
   
   lappend TestedDirs $Arg
  }
 }
   
 if {[llength $TestedDirs] > 0} \
 {
  set VAR($Path,SelectedDir) $TestedDirs
  focus $VAR($Path,Parent)
  destroy .$Path
  set VAR($Path,Stop) 1
 }
}


#---create a new directory
proc ::FR_dirBox::NewDir {Path} \
{
 variable VAR
  
 set Dir [::FR_popupWindow::create -parent .$Path -initialinput $VAR($Path,TmpDir) -description [msgcat::mc "Input your new directory:"]]
  
 if {$Dir eq ""} {return 1}
  
 set Dir [file normalize $Dir]
  
 if {[file exists $Dir]} \
 {
  ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "This directory already exists."]  
  ::FR_dirBox::DirTreeRefresh $Path $Dir
  return 1
 }
   
 if { ![catch { file mkdir $Dir } ReturnContent] } \
 {
  ::FR_dirBox::DirTreeRefresh $Path $Dir
 } else \
 {
  ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "I can't create this directory!\nThe reason is:\n\n%s" $ReturnContent]
  ::FR_dirBox::DirTreeRefresh $Path
 }  
}
 
 
#---change directly in another directory
proc ::FR_dirBox::ChangeDir {Path} \
{
 variable VAR
  
 set Dir [::FR_popupWindow::create -parent .$Path -initialinput $VAR($Path,TmpDir) -description [msgcat::mc "Input your new directory:"]]
  
 if {$Dir eq ""} {return 1}
  
 set Dir [file normalize $Dir]
  
 if {[file isdirectory $Dir]} \
 {
  ::FR_dirBox::DirTreeRefresh $Path $Dir
 } else \
 {
  ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "%s isn't a valid directory" $Dir]
 }
}

 
#---delete directories
proc ::FR_dirBox::DeleteDir {Path} \
{
 variable VAR
   
 set Index [.$VAR($Path,dirtree).dirtree curselection]
  
 if {$Index eq ""} \
 {
  ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "Please select first a directory."]
   
  return 1
 }
  
 set Selected ""
     
 foreach i $Index \
 {
  lappend Selected [file join $VAR($Path,TmpDir) [.$VAR($Path,dirtree).dirtree get $i]]
 }
   
 if { [::FR_messageBox::Create -message [msgcat::mc "Are you sure to delete the follow directories:\n\n%s" $Selected] -title [msgcat::mc "Are you sure?"] -type Info -parent .$Path -button [list [msgcat::mc "Yes"] [msgcat::mc "No"]]] } \
 {
  foreach i $Selected \
  {
   if { [catch { file delete -force $i } ReturnContent] } \
   {
    ::FR_messageBox::Create -type Error -parent .$Path -message [msgcat::mc "I can't delete this directory!\nThe reason is:\n\n%s" $ReturnContent]
   } 
  }
    
  ::FR_dirBox::DirTreeRefresh $Path
 }
}   


#---refresh the entry box after selecting
proc ::FR_dirBox::RefreshEntry {Path Widget} \
{
 variable VAR
  
 set Selected ""
  
 set Index [$Widget curselection]
 if {$Index eq ""} {return}
  
 if { [lindex $Index 0] eq 0 } \
 {
  if {[$Widget get 0] eq ".." } {set Index [lrange $Index 1 end]}
 }
   
 foreach i $Index \
 {
  lappend Selected [file join $VAR($Path,TmpDir) [$Widget get $i]]
 }
     
 if {[llength $Selected] > 0} {::FR_dirBox::InsertText $Path $Selected}
}   


#---the reaction after pressing the cancel button
proc ::FR_dirBox::CancelButton {Path} \
{
 variable VAR
  
 focus $VAR($Path,Parent)
 destroy .$Path
 set VAR($Path,SelectedDir) ""
 set VAR($Path,Stop) 1
}


#---the ballonhelp
proc ::FR_dirBox::BalloonHelp {Path} \
{
 variable VAR
 
 set ::FR_balloonHelp::wl(.$VAR($Path,dirtree).dirtree) [msgcat::mc "With a right-mouse-button-klick you get a menu with some directory functions"]
 set ::FR_balloonHelp::wl(.$VAR($Path,entry).entry) [msgcat::mc "This is the list with your chosen directories"] 

 if {$VAR($Path,BalloonHelp)} {::FR_balloonHelp::create}
}


#---------------------
#---the main procedure
#---------------------

proc ::FR_dirBox::Create {args} \
{
 global env
 variable VAR

 #---first we generate a toplevel name who doesn't exist yet
 set i 1
 while {[winfo exists .filebox_dialog$i]} {incr i}
 set Path "filebox_dialog$i"
 
 #---some presettings    
 set VAR($Path,Stop) 0
 set VAR($Path,TmpDir) "$env(HOME)"
 set VAR($Path,HiddenDirs) 0
 set VAR($Path,HiddenFiles) 0
 set VAR($Path,Title) [msgcat::mc "Please choose a directory:"]
 set VAR($Path,Parent) .
 set VAR($Path,Permission) "readable" 
 set VAR($Path,Multiple) 0
 set VAR($Path,FileType) ""
 set VAR($Path,BalloonHelp) 0
  
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
  set Arg [lindex $args $i]
  switch -glob -- $Arg \
   {
    -initialdir        {set VAR($Path,TmpDir) [lindex $args [expr {$i+1}]]} \
    -view_hidden_files {set VAR($Path,HiddenFiles) [lindex $args [expr {$i+1}]]} \
    -view_hidden_dirs  {set VAR($Path,HiddenDirs) [lindex $args [expr {$i+1}]]} \
    -title             {set VAR($Path,Title) [lindex $args [expr {$i+1}]]} \
    -parent            {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
    -permission        {set VAR($Path,Permission) [lindex $args [expr {$i+1}]]} \
    -multiple          {set VAR($Path,Multiple) [lindex $args [expr {$i+1}]]} \
    -filetype          {set VAR($Path,FileType) [lindex $args [expr {$i+1}]]} \
    -balloonhelp       {set VAR($Path,BalloonHelp) [lindex $args [expr {$i+1}]]} \
    --                 {break} \
    *                  {puts "Unknown option: $Arg"; exit 1}
   }
  }

 #---we generate a new toplevel window
 toplevel .$Path
 
 #---we set the windowsname and hide it
 wm title .$Path "$VAR($Path,Title)"
 wm withdraw .$Path
  
 #---we check the arguments
 #---if a wrong directory set -> we set it to HOME
 if {![file isdirectory $VAR($Path,TmpDir)]} \
 {
  set VAR($Path,TmpDir) "$env(HOME)"
 }
 
  #---if no file type set -> we set it to "All files"
 if {$VAR($Path,FileType) eq ""} \
 {
  set VAR($Path,FileType) [list [msgcat::mc "All files"] "*.*"]
 }
 
 #---if no parent window set -> we set it to "."
 if {![winfo exists $VAR($Path,Parent)]} \
 {
  set VAR($Path,Parent) .
 }

 #---we set the selectmode of the file
 if {$VAR($Path,Multiple)} \
 {
  set VAR($Path,ListBoxMode) "extended"
 } else \
 {
  set VAR($Path,ListBoxMode) "browse"
 }
     
 #---the frame for the buttons
 frame .$Path.frame2 -relief flat
 labelframe .$Path.frame3 -relief groove -text [msgcat::mc "Chosen directory"]
 labelframe .$Path.frame4 -relief groove -text [msgcat::mc "Directory tree"]
 labelframe .$Path.frame5 -relief groove -text [msgcat::mc "Files"] 
 #-foreground gray
 
 pack  .$Path.frame2 -side bottom -fill x
 pack  .$Path.frame3 -side bottom -fill x -padx 5 -pady 5
 pack  .$Path.frame4 -side left -padx 5 -pady 5 -expand yes -fill both
 pack  .$Path.frame5 -side left -padx 5 -pady 5 -expand yes -fill both

  
 #---We set a variable for each frame - so we can change the order of the frames later easier
 set VAR($Path,botton) "$Path.frame2"
 set VAR($Path,entry) "$Path.frame3"
 set VAR($Path,dirtree) "$Path.frame4"
 set VAR($Path,files) "$Path.frame5"
 
 listbox .$VAR($Path,dirtree).dirtree -background white -activestyle none -selectmode $VAR($Path,ListBoxMode) \
 -yscroll ".$VAR($Path,dirtree).yscrollbar set" -xscroll ".$VAR($Path,dirtree).xscrollbar set"
 
 scrollbar .$VAR($Path,dirtree).yscrollbar -orient vertical -command ".$VAR($Path,dirtree).dirtree yview"
 scrollbar .$VAR($Path,dirtree).xscrollbar -orient horizontal -command ".$VAR($Path,dirtree).dirtree xview"
 
 grid .$VAR($Path,dirtree).dirtree -row 0 -column 0 -sticky nsew -padx {5 0} -pady {5 0}
 grid .$VAR($Path,dirtree).yscrollbar -row 0 -column 1 -sticky ns -padx {0 5} -pady {5 0}
 grid .$VAR($Path,dirtree).xscrollbar -row 1 -column 0 -sticky ew -padx {5 0} -pady {0 5}

 grid columnconfigure .$VAR($Path,dirtree) 0 -weight 1
 grid rowconfigure .$VAR($Path,dirtree) 0 -weight 1

 #---a menu to delete or create new dirs
 menu .$VAR($Path,dirtree).menu -tearoff 0
 .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go to home directory"] -command [list ::FR_dirBox::DirTreeRefresh $Path $env(HOME)]
 .$VAR($Path,dirtree).menu add separator
 .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go to / -directory"] -command [list ::FR_dirBox::DirTreeRefresh $Path /]
 .$VAR($Path,dirtree).menu add separator 
 .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Go direct in another directory"] -command [list ::FR_dirBox::ChangeDir $Path] 
 .$VAR($Path,dirtree).menu add command -label "" -command {}
 .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Add a new directory"] -command [list ::FR_dirBox::NewDir $Path]
 .$VAR($Path,dirtree).menu add separator
 .$VAR($Path,dirtree).menu add command -label [msgcat::mc "Delete a directory"] -command [list ::FR_dirBox::DeleteDir $Path]


 bind .$VAR($Path,dirtree).dirtree <<ListboxSelect>> [list ::FR_dirBox::RefreshEntry $Path %W]   
 bind .$VAR($Path,dirtree).dirtree <Double-Button-1> [list ::FR_dirBox::DirTreeSelection $Path %W]
 bind .$VAR($Path,dirtree).dirtree <Button-3> [list tk_popup .$VAR($Path,dirtree).menu %X %Y]
 
 #---the listbox for the files
 listbox .$VAR($Path,files).files -background white -activestyle none -state disable \
 -yscroll ".$VAR($Path,files).yscrollbar set" -xscroll ".$VAR($Path,files).xscrollbar set"
 
 scrollbar .$VAR($Path,files).yscrollbar -orient vertical -command ".$VAR($Path,files).files yview"
 scrollbar .$VAR($Path,files).xscrollbar -orient horizontal -command ".$VAR($Path,files).files xview"
 
 grid .$VAR($Path,files).files -row 0 -column 0 -sticky nsew -padx {5 0} -pady {5 0}
 grid .$VAR($Path,files).yscrollbar -row 0 -column 1 -sticky ns -padx {0 5} -pady {5 0}
 grid .$VAR($Path,files).xscrollbar -row 1 -column 0 -sticky ew -padx {5 0} -pady {0 5}

 grid columnconfigure .$VAR($Path,files) 0 -weight 1
 grid rowconfigure .$VAR($Path,files) 0 -weight 1


 #---The ok and chancel buttons
 set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
 
 button .$VAR($Path,botton).ok -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::FR_dirBox::EntryCheck $Path]
 button .$VAR($Path,botton).cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::FR_dirBox::CancelButton $Path]
  
 pack .$VAR($Path,botton).ok -side left -padx 5 -pady 5 -fill x -expand yes
 pack .$VAR($Path,botton).cancel -side left -padx 5 -pady 5 -fill x -expand yes
 
 bind .$Path <KeyPress-Escape> [list .$VAR($Path,botton).cancel invoke]
 bind .$Path <KeyPress-Return> [list .$VAR($Path,botton).ok invoke]
 
 
 #---the entry-box (no input only for controlling)
 entry .$VAR($Path,entry).entry -state disable -disabledforeground black -disabledbackground white -xscrollcommand ".$VAR($Path,entry).mkdir_xsbar set" 
 scrollbar .$VAR($Path,entry).mkdir_xsbar -orient horizontal -command ".$VAR($Path,entry).entry xview"
  
 grid .$VAR($Path,entry).entry -row 0 -column 0 -sticky ew -padx 5
 grid .$VAR($Path,entry).mkdir_xsbar -row 1 -column 0 -sticky nsew -pady {0 5} -padx 5
  
 grid columnconfigure .$VAR($Path,entry) 0 -weight 1
 
 
 ::FR_dirBox::DirTreeRefresh $Path $VAR($Path,TmpDir)
 
 eval ::FR_general::CenterWindow .$Path $VAR($Path,Parent) 1 450 300
 wm deiconify .$Path
 focus .$Path
 
 ::FR_dirBox::BalloonHelp $Path

 wm protocol .$Path WM_DELETE_WINDOW [list .$VAR($Path,botton).cancel invoke]
 
 #---We stop the procedure and wait until the return value is ready
 vwait ::FR_dirBox::VAR($Path,Stop)
 return "$VAR($Path,SelectedDir)"
}

