# converter.tcl
#
#

#---the main proc for the converter
proc ::tk_cdconverter::ConverterMain {} \
{
  variable VAR
   
  if {![info exists VAR(GenreList)]} \
  {
    ::tk_cdconverter::LibGenerateGenreList
  }
  
  ::tk_cdconverter::ConverterGui
  
  ::tk_cdconverter::LibBitrateButtonState $VAR(converterframe).bitrate
}


#---after a switch to this program part we make some presettings
proc ::tk_cdconverter::ConverterClearGui {} \
{
  variable VAR
  
  ::FR_general::MakeDisable .$VAR(converterframe).start
}


#---we create and place all gui elements
proc ::tk_cdconverter::ConverterGui {} \
{
  variable VAR; variable CONFIG
  
  # the widgets
  label .$VAR(converterframe).inputdir_comment -text [msgcat::mc "Get the orginal files from:"]
  button .$VAR(converterframe).inputdir -text [msgcat::mc "Open"] -command {::tk_cdconverter::ConverterInputDir}
  
  checkbutton .$VAR(converterframe).delete_orginal -text [msgcat::mc "Delete the orginal files in the input-directory after converting?"] -variable ::tk_cdconverter::CONFIG(DeleteInputFiles) -pady 10
  
  label .$VAR(converterframe).outputdir_comment -text [msgcat::mc "Save the converted files to:"] -pady 10
  button .$VAR(converterframe).outputdir -text [msgcat::mc "Save in"] -command {::tk_cdconverter::ConverterOutputDir}
  
  label .$VAR(converterframe).from_comment -pady 10 -text [msgcat::mc "Convert from format:"]
  label .$VAR(converterframe).to_comment -pady 10 -text [msgcat::mc "To format:"]

  ::FR_comboBox::cb .$VAR(converterframe).from -variable ::tk_cdconverter::CONFIG(InputFormat)
  ::FR_comboBox::cb .$VAR(converterframe).to -variable ::tk_cdconverter::CONFIG(OutputFormat)

  ::FR_comboBox::cb .$VAR(converterframe).from -listbox_content $VAR(DecoderList) -entry_width auto -default_value $::tk_cdconverter::CONFIG(InputFormat) -command {::tk_cdconverter::CheckInputDirContent}
  ::FR_comboBox::cb .$VAR(converterframe).to -listbox_content $VAR(EncoderList) -entry_width auto -default_value $::tk_cdconverter::CONFIG(OutputFormat) -command [list ::tk_cdconverter::LibBitrateButtonState $VAR(converterframe).bitrate]
  
  label .$VAR(converterframe).bitrate_comment -pady 10 -text [msgcat::mc "Select the desired bitrate (mp3/ogg-vorbis only):"]

  ::FR_comboBox::cb .$VAR(converterframe).bitrate -variable ::tk_cdconverter::CONFIG(BitRate)
  ::FR_comboBox::cb .$VAR(converterframe).bitrate -listbox_content [list 64 128 192 256] -entry_width 3 -default_value $::tk_cdconverter::CONFIG(BitRate)
  
  button .$VAR(converterframe).start -text [msgcat::mc "Start"] -state disabled -command {::tk_cdconverter::ConverterStartButton}
  
  # the placement
  set row 0
  
  ::FR_general::Topic $VAR(converterframe) $row [msgcat::mc "Input directory:"] 2
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(converterframe).inputdir_comment 0 w .$VAR(converterframe).inputdir 1 ew
  eval ::FR_general::PackHelp_2 [incr row] .$VAR(converterframe).delete_orginal 0 2 w
  ::FR_general::Line $VAR(converterframe) [incr row] 2

  ::FR_general::Topic $VAR(converterframe) [incr row] [msgcat::mc "Output directory:"] 2
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(converterframe).outputdir_comment 0 w .$VAR(converterframe).outputdir 1 ew
  ::FR_general::Line $VAR(converterframe) [incr row] 2 

  eval ::FR_general::PackHelp_1 [incr row] .$VAR(converterframe).from_comment 0 w .$VAR(converterframe).from 1 ew 
  eval ::FR_general::PackHelp_1 [incr row] .$VAR(converterframe).to_comment 0 w .$VAR(converterframe).to 1 ew
  ::FR_general::Line $VAR(converterframe) [incr row] 2

  eval ::FR_general::PackHelp_1 [incr row] .$VAR(converterframe).bitrate_comment 0 w .$VAR(converterframe).bitrate 1 e 
  ::FR_general::Line $VAR(converterframe) [incr row] 2
 
  ::FR_general::PadFrame $VAR(converterframe) [incr row] 2
 
  eval ::FR_general::PackHelp_2 [incr row] .$VAR(converterframe).start 0 2 ew

  grid columnconfigure .$VAR(converterframe) 0 -weight 1
  
  # the balloon help
  ::FR_balloonHelp::AddHelp ".$VAR(converterframe).inputdir" [msgcat::mc "This is the directory with your files which you want to convert"]
  ::FR_balloonHelp::AddHelp ".$VAR(converterframe).outputdir" [msgcat::mc "This is the directory in which you want to store the converted files"]
  ::FR_balloonHelp::AddHelp ".$VAR(converterframe).from" [msgcat::mc "Here you can choose the kind of files (formats) which you want to convert"]
  ::FR_balloonHelp::AddHelp ".$VAR(converterframe).to" [msgcat::mc "This is the codec which is used for your output files"]
  ::FR_balloonHelp::AddHelp ".$VAR(converterframe).start" [msgcat::mc "Please keep in mind - the quality of your songs will be worse if you convert from a codec with losses (e.g. mp3 or ogg-vorbis). The high-end solution would be: to store your music collection in flac and convert mp3 or ogg-vorbis files from it when you need this formats e.g. for your portable player."]
  
  ::tk_cdconverter::LibBalloonHelpStatus
}


#---load the input directory
proc ::tk_cdconverter::ConverterInputDir {} \
{
  variable CONFIG; variable VAR
 
  set InputDir [::FR_dirBox::Create -initialdir $CONFIG(InputDir) -title [msgcat::mc "Choose the directory with your orginal files:"] -permission "writeable" -filetype $VAR(DecoderFileExtension)]

  if {$InputDir ne ""} \
  {
    set CONFIG(InputDir) $InputDir

    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "The input directory is set to %s" $InputDir]
    
    ::tk_cdconverter::CheckInputDirContent
  }
}


#---load the output directory
proc ::tk_cdconverter::ConverterOutputDir {} \
{
  variable CONFIG; variable VAR

  set OutputDir [::FR_dirBox::Create -initialdir $CONFIG(OutputDir) -title [msgcat::mc "Choose the output directory for your converted files:"] -parent .$VAR(converterframe) -permission "writable"  -filetype $VAR(EncoderFileExtension)]

  if {$OutputDir ne ""} \
  {
    set CONFIG(OutputDir) $OutputDir

    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "The output directory is set to %s" $OutputDir]

    ::FR_general::MakeNormal .$VAR(converterframe).start
  }
}


#---the start button
proc ::tk_cdconverter::ConverterStartButton {} \
{
  variable CONFIG
 
  if {$CONFIG(InputFormat) eq $CONFIG(OutputFormat)} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "You have chosen the same format for the input and the output files. So, no conversation is necessary."]
    return
  }
  
  if {[::tk_cdconverter::ConverterStart]} \
  {
    ::FR_messageBox::Create -message [msgcat::mc "An error occured during converting. Please check the message window for further information."] -title [msgcat::mc "End of job"] -type Error
  } \
  else \
  {
    ::FR_messageBox::Create -message [msgcat::mc "All files have been successful converted."] -title [msgcat::mc "End of job"] -type Info
  }  
}


#---the main loop, convertes the input files into a (wav-)tmp-file
proc ::tk_cdconverter::ConverterStart {} \
{
  variable CONFIG; variable VAR

  set VAR(DeleteAllOutputfiles) 0

  set InputFiles [::tk_cdconverter::LibCheckDirContent $CONFIG(InputDir) $CONFIG(InputFormat)]

  if {$InputFiles eq ""} \
  {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "There are no %s-Files in %s." $CONFIG(InputFormat) $CONFIG(InputDir)]
    return 1
  }
 
  set TmpFile [file join $CONFIG(OutputDir) tmp.wav]
 
  if {[file exists $TmpFile]} \
  {
    catch {file delete $TmpFile}
  }
   
  set Index 1
  
  # we hope the best... :o)
  set ReturnValue 0 
 
  foreach File $InputFiles \
  {
    ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "Now, we convert the file \"%s\". This takes a while...!" $File]
    
    set OutputFile [::tk_cdconverter::LibOutputFileName $File $CONFIG(OutputDir) $CONFIG(OutputFormat)]
    
    if {![::tk_cdconverter::LibCheckOutputFile $OutputFile]} \
    {
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I can't create: %s. This file already exists and you didn't give the permission to overwrite it." $OutputFile]
      
      set ReturnValue 1
      break
    }
        
    set DecodeReturn [::tk_cdconverter::LibCompressed2Wav [file join $CONFIG(InputDir) $File] $TmpFile $CONFIG(InputFormat)]
    
    if {$DecodeReturn == 0} \
    {
      set EncodeResult [::tk_cdconverter::LibWav2Compressed $TmpFile $OutputFile $CONFIG(OutputFormat) $CONFIG(BitRate)]
      
    } \
    else \
    {
      # an error occured
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't decode the file %s. The reason for this was:\n%s" $File $DecodeReturn]
      
      set ReturnValue 1
      break
    }
  
    if {$EncodeResult != 0} \
    {
      # an error occured
      ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't encode the file %s. The reason for this was:\n%s" $OutputFile $EncodeResult]
      
      set ReturnValue 1
      break
    }
  
    set Taglist [::tk_cdconverter::LibReadTag [file join $CONFIG(InputDir) $File] $CONFIG(InputFormat)]
    
    # at the moment we don't use the track information of the read command
    set Taglist [lrange $Taglist 0 end-1]
    
    # we add our own index at the end of the taglist
    lappend Taglist $Index
    
    eval ::tk_cdconverter::LibWriteTag $Taglist $OutputFile $CONFIG(OutputFormat)
    
    if {$CONFIG(DeleteInputFiles)} \
    {
      catch {file delete [file join $CONFIG(InputDir) $File]}
      
      # if the input file a wave file: a file with the tag informations (e.g. Song01Tags.txt) could
      # also exist -> we delete this file also
      if {$CONFIG(InputFormat) eq "Wav"} \
      {
        set TxtFile [file rootname $File]
        append TxtFile "Tags.txt"
        set TxtFile [file join $CONFIG(InputDir) $TxtFile]
        
        if {[file exists $TxtFile]} \
        {
          catch {file delete $TxtFile}
        }
      }
    }
    
    incr Index
  }
  
  if {[file exists $TmpFile]} \
  {
    catch {file delete $TmpFile}
  }
  
  return $ReturnValue
}


#---puts out a message if a given file type isn't in the directory
proc ::tk_cdconverter::CheckInputDirContent {} \
{
 variable CONFIG
 
 set InputFiles [::tk_cdconverter::LibCheckDirContent $CONFIG(InputDir) $CONFIG(InputFormat)]
 
 if {$InputFiles eq ""} \
 {
   ::FR_msgWindow::Add -format Info -msg [msgcat::mc "There aren't any %s-Files in the directory: %s at the moment." $CONFIG(InputFormat) $CONFIG(InputDir)]
 } \
 else \
 {
   ::FR_msgWindow::Add -format Info -msg [msgcat::mc "We found %s %s-Files in %s." [llength $InputFiles] $CONFIG(InputFormat) $CONFIG(InputDir)]
 }
}


::tk_cdconverter::ConverterMain
